  %Code associated with the paper: J. Krissansen-Totton,
  %D. S. Bergsman and D.C. Catling (2016) "On Detecting Biospheres
  %from Thermodynamic Disequilibrium in Planetary Atmospheres",
  %Astrobiology 16, 39-67, 2016. 
  %As a matter of courtesy, we request that people using this code
  %please cite Krissansen-Totton et al. (2016). In the interest of
  %an "open source" approach, we also request that authors who use
  %and modify the code, please send a copy of papers and modified
  %code to senior author, Prof. David Catling  (dcatling@uw.edu ).
  % This work was funded by NASA's Astrobiology/Exobiology Program.
function n=Main_code_Mars
    
    format('long') %ensures all significant figures outputted
    global R T P g obsern fug names; %define important variables as global for use in subsidiary functions
    R = 8.3145; %Universal gas constant (Joules/Kelvin/mol)
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% BEGIN INPUTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    T = 214; %Average surface temperature on Mars (Kelvin)
    P = .006; %Average surface pressure on Mars (bar)
    
    %option for gas fugacity coefficients
    %fug='y' for inclusion of gas fugacities, 'n' for ideal gas behaviour
    fug='y'; 
    
    % IMPORTANT: Make sure there is one space after each species (e.g. type
    % 'N2 ' and not 'N2' to ensure search function finds the correct species)
    names={'CO2 ','N2 ','Ar ','O2 ','CO ','H2O ','NO ','Ne ','Kr ','Xe ','O3 ','NO2 ','H2 ','H2O2 '};
    n=[.9597 .0189 .0193 .00146 .000557 .0002 .1e-8 .0000025 .0000003 .00000008 40e-8 1e-30 .000015 4e-8]'; %mixing ratios for each species
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% END INPUTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Call 'gibbs' function that obtains standard Gibbs free energy of
    % formation for each molecular species at temperature, T.
    g=gibbs(names,T);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Optional correction:
    % change Argon abundance to ensure mixing ratios sum to exactly 1.
    missing = 1-sum(n,'double');
    n(3)=n(3)+missing;
    % comment this section out if code is being modified for other planets
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    display('Initial mixing ratios')
    obsern=n %store and print initial abundances as "obsern"
    
    %Matrix with entries specifying the number of atoms of each element in
    %each molecular species. Columns are the molecular species in "n", rows
    %are the atmoms. Atoms and molecules have been labeled for clarity.
    %       CO2    N2    Ar    O2    CO    H2O   NO    Ne    Kr    Xe   O3    NO2   H2   H2O2
     a= [    1     0     0     0     1     0     0     0     0     0     0     0     0     0; %C
             2     0     0     2     1     1     1     0     0     0     3     2     0     2; %O
             0     2     0     0     0     0     1     0     0     0     0     1     0     0; %N
             0     0     1     0     0     0     0     0     0     0     0     0     0     0; %Ar
             0     0     0     0     0     2     0     0     0     0     0     0     2     2; %H
             0     0     0     0     0     0     0     1     0     0     0     0     0     0; %Ne
             0     0     0     0     0     0     0     0     1     0     0     0     0     0; %Kr
             0     0     0     0     0     0     0     0     0     1     0     0     0     0]; %Xe
    %%% alternatively, call function that calculates 'a' matrix automatically
    % rather than manually input matrix:
    elements=makeAs(names);
    a = cell2mat(elements(2:end,:))';
    
    % initial conditions for atom conservation constraint
    b = a*n;
    
    % Tolerance variables for optimization function:
    tol = 1e-30; tol2 = 1e-60;
    
    %Calculate Gibbs energy of initial (observed state)
    G1 = totalGibbs(n);
    
    %Find equilibrium state, n, and calculate Gibbs energy of equilibrium
    %state, G2.
    [n,G2,exitflag]=fmincon(@totalGibbs,n,[],[],a,b,repmat(1e-30,size(n)),repmat(Inf,size(n)),[],optimset('TolX',tol2,'TolFun',tol,'FunValCheck','on','Algorithm','interior-point'));
    display('Final abundances')
    n
    
    % Calculate available Gibbs free energy (difference between final and
    % inital Gibbs free energy). Because initial abundances were mixing
    % ratios, then Phi has units of joules per mole of atmosphere
    display('Gibbs energy change')
    Phi = G2-G1
  
    %Call function that plots initial and final abundaces with histogram
    hist_plot(obsern,n);
    
end

% Subsidiary function for calculating Gibbs energy of mixture of gases
function fun = totalGibbs(n)
    global R T g P fug names; %load global variables for use within this subsidiary function
   
    if fug=='y'  
    % Call subsidiary function that calculates fugacity coefficients for each molecular species
        lnPhi = fugCoef(T,P,names,n'); %no spaces after names here
        %%% CHECK WHETHER SHOULD USE strtrim(names) instead
    elseif fug=='n'
        lnPhi=0; %gas fugacity coefficients=1 for ideal gases so log(fugacity_coef)=0
    end
    
    fun = R*T*n'*(g/R/T+log(n/sum(n,'double'))+log(P)+lnPhi); %% Gibbs energy function for gaseous system
    
end

% Subsidiary function for plotting histogram
function hist_plot(obsern,n)
    global names
    figure()
    cutoff=1e-10; %minimum mixing ratio for plotting
    
    % This loop sets all trace species with abundances less than the cutoff to equal the cutoff
    % This is a purely aesthetic change to make the figure clear
    for i = 1 : length(n)
        if n(i)<cutoff
            n(i)=cutoff;
        end
        if obsern(i)<cutoff
            obsern(i)=cutoff;
        end
    end
    
    % Define greyscale color scheme for figure
    mycolor=[0 0 0;0.2 0.2 0.2;0.7 0.7 0.7];
    colormap(mycolor)
    
    % create and plot histogram
    set(gca,'YScale','log','fontsize',10)
    hold on;
    str=strtrim(names);
    bar([obsern,n],'basevalue',cutoff)
    set(gca,'XTicklabel',str,'Xtick',1:numel(str))
    ylabel('Mixing ratio')
    legend('observed','equilibrium')
    title('Mars')
    grid on
    
return
end